from __future__ import print_function
import sys
import time
from datetime import datetime

import boto
from boto.ec2.autoscale import (LaunchConfiguration, AutoScalingGroup,
    ScalingPolicy)
from fabric.decorators import task
from fabric.operations import puts
from fabric.utils import abort


@task
def create_ami(instance_id, name):
    """
    Create AMI image from specified instance

    The instance needs to be shutdown before the creation begin.
    """
    image_name = "{0}_{1}".format(name, datetime.now().strftime("%Y%m%d-%H%M"))

    conn = boto.connect_ec2()
    image_id = conn.create_image(instance_id=instance_id, name=image_name)
    puts("Creating AMI {0} for instance {1}".format(image_name, image_id))

    while True:
        puts('.', end='')
        sys.stdout.flush()

        image = conn.get_image(image_id)
        if image.state == 'available':
            break
        if image.state == "failed":
            abort("Error creating AMI for {0}".format(image_id))
        time.sleep(5.0)

    puts("\nImage {0} created".format(image_name))
    return image_id


def create_scaling_policy(conn, name, as_name, scaling_adjustment, cooldown):
    scaling_policy = ScalingPolicy(name=name, as_name=as_name,
                                   adjustment_type='ChangeInCapacity',
                                   scaling_adjustment=scaling_adjustment,
                                   cooldown=cooldown)
    conn.create_scaling_policy(scaling_policy)


@task
def setup_autoscale(name, ami_id, key_name, security_groups, load_balancers,
        instance_type='m1.micro', availability_zones=['us-east-1b'],
        min_instances=1, max_instances=12,
        sp_up_adjustment=1, sp_up_cooldown=180,
        sp_down_adjustment=-1, sp_down_cooldown=180,
        instance_monitoring=True
    ):
    """
    Configure AutoScaling for Amazon EC2 instances

        `name`: name used to identify the autoscale group
        `ami_id`: AMI ID from instances will be generated
        `key_name`: name of the SSH key which will have access to the instance
        `security_groups`: list of security groups to associate with each
                           created instance
        `instance_type`: type of the instance that will be launched
                         (see http://aws.amazon.com/ec2/instance-types/)
        `availability_zones`: in which zones instances can be launched. This
                              must match with zones configured in ELB.
        `min_instances`: minimal number of instances that must be running
        `max_instances`: maximum number of instance that must be running
        `sp_up_adjustment`: sets the number of instances to launch on the up
                            scaling policy trigger
        `sp_down_adjustment`: sets the number of instances to kill on the down
                            scaling policy trigger
    """
    launch_config = "{0}_{1}".format(name, datetime.now().strftime("%Y%m%d-%H%M"))
    group_name = '{0}-as-group'.format(name)

    sp_up_name = '{0}-scaling-up'.format(name)
    sp_down_name = '{0}-scaling-down'.format(name)

    conn_as = boto.connect_autoscale()
    lc = LaunchConfiguration(name=launch_config, image_id=ami_id,
                             key_name=key_name,
                             security_groups=security_groups,
                             instance_type=instance_type,
                             instance_monitoring=instance_monitoring)
    conn_as.create_launch_configuration(lc)

    ag = AutoScalingGroup(group_name=group_name, load_balancers=load_balancers,
                          availability_zones=availability_zones,
                          launch_config=launch_config,
                          min_size=min_instances, max_size=max_instances)
    conn_as.create_auto_scaling_group(ag)

    create_scaling_policy(conn_as, sp_up_name, group_name, sp_up_adjustment, sp_up_cooldown)
    create_scaling_policy(conn_as, sp_down_name, group_name, sp_down_adjustment, sp_down_cooldown)


def get_autoscaling_instances(elb_name):
    """Get public DNS from autoscaling instances registered with ELB `elb_name`."""
    elb_conn = boto.connect_elb()
    loadbalancers = elb_conn.get_all_load_balancers([elb_name])

    instances_ids = []
    for instance in loadbalancers[0].instances:
        instances_ids.append(instance.id)

    ec2_conn = boto.connect_ec2()
    reservations = ec2_conn.get_all_instances(instances_ids)

    ec2_instances = []
    for reservation in reservations:
        ec2_instances.append(reservation.instances[0].public_dns_name)
    return ec2_instances


@task
def replace_launch_config(name, image_id=None, key_name=None,
        security_groups=None, instance_type=None, instance_monitoring=None):
    """Replace launch configuration associated with auto scaling group."""

    conn = boto.connect_autoscale()
    as_group = conn.get_all_groups(['{0}-as-group'.format(name)], max_records=1)[0]
    orig_launch_config_name = as_group.launch_config_name

    orig_launch_config = conn.get_all_launch_configurations(names=[orig_launch_config_name],
                                                            max_records=1)[0]

    if not image_id:
        image_id = orig_launch_config.image_id
    if not key_name:
        key_name = orig_launch_config.key_name
    if not security_groups:
        security_groups = orig_launch_config.security_groups
    if not instance_type:
        instance_type = orig_launch_config.instance_type
    if not instance_monitoring:
        instance_monitoring = orig_launch_config.instance_monitoring

    config_name = "{0}_{1}".format(name, datetime.now().strftime("%Y%m%d%H%M"))

    puts("Replace launch config {0} with {1}".format(orig_launch_config_name, config_name))

    new_launch_config = LaunchConfiguration(name=config_name, image_id=image_id,
                                            key_name=key_name,
                                            security_groups=security_groups,
                                            instance_type=instance_type,
                                            instance_monitoring=instance_monitoring)
    conn.create_launch_configuration(new_launch_config)

    min_size = int(as_group.min_size)
    max_size = int(as_group.max_size)

    as_group.launch_config_name = config_name
    # We need to setup min & max size again
    as_group.min_size = min_size
    as_group.max_size = max_size
    as_group.update()

    orig_launch_config.delete()
    return orig_launch_config_name, config_name


@task
def update_autoscale(instance_id, name):
    """Create AMI image from `instance_id` and update autoscale configuration"""
    image_id = create_ami(instance_id, name)
    replace_launch_config(name, image_id)



# coding: utf-8
import cuisine
import fabric.api as fab

import fabix


def install():
    fabix.system.apt_import_pubkey('7F0CEB10')
    fabix.system.apt_add_repository('10gen', 'dist', 'http://downloads-distro.mongodb.org/repo/ubuntu-upstart')
    cuisine.package_update()
    cuisine.package_upgrade()
    cuisine.package_install(['mongodb-10gen', 'ntp', 'lvm2'])


def change_datadir(datadir):
    with cuisine.mode_sudo():
        cuisine.dir_ensure(datadir, recursive=True)
    fab.sudo("chown -R mongodb:mongodb {}".format(datadir))
    fab.sudo("sed -i 's/^dbpath=\/var\/lib\/mongodb/dbpath={}/' /etc/mongodb.conf".format(datadir.replace('/', '\/')))
    fabix.system.upstart.restart('mongodb', force_start=True)



# coding: utf-8
import os

import fabric.api as fab


def install(filename):
    """Install upstart script on remote host.

    filename will be uploaded to /etc/init on remote server."""
    name = os.path.basename(filename)
    fab.put(filename, '/etc/init/{0}'.format(name), use_sudo=True)


@fab.task
def start(name, **kwargs):
    """Start specified upstart job.

    If **kwargs is specified, then they will be passed on command line
    as variables to upstart script.

    For example, to start a service called foo:
        `fab start:foo`
    If you want to pass a variable PORT to upstart script:
        `fab start:foo,PORT=12345`
    """
    args = ' '.join(['{}={}'.format(k, v) for k, v in kwargs.iteritems()])
    fab.sudo("start {name} {args}".format(name=name, args=args))


@fab.task
def stop(name, **kwargs):
    """Stop specified upstart job.

    If **kwargs is specified, then they will be passed on command line
    as variables to upstart script.

    For example, to stop a service called foo:
        `fab stop:foo`
    If you want to pass a variable PORT to upstart script:
        `fab stop:foo,PORT=12345`

    If you pass any variable to upstart on initialization, the same variables
    must be passed to stop.
    """
    args = ' '.join(['{}={}'.format(k, v) for k, v in kwargs.iteritems()])
    fab.sudo("stop {name} {args}".format(name=name, args=args))


@fab.task
def reload(name, **kwargs):
    """Reload specified upstart job.

    If **kwargs is specified, then they will be passed on command line
    as variables to upstart script.

    For example, to reload a service called foo:
        `fab reload:foo`
    If you want to pass a variable PORT to upstart script:
        `fab reload:foo,PORT=12345`

    If you pass any variable to upstart on initialization, the same variables
    must be passed to reload.
    """
    args = ' '.join(['{}={}'.format(k, v) for k, v in kwargs.iteritems()])
    fab.sudo("reload {name} {args}".format(name=name, args=args))


@fab.task
def restart(name, force_start=True, **kwargs):
    """Restart specified upstart job.

    If force_start is true, then the job will be started if it isn't running.

    If **kwargs is specified, then they will be passed on command line
    as variables to upstart script.

    For example, to restart a service called foo:
        `fab restart:foo`
    If you want to pass a variable PORT to upstart script:
        `fab reload:foo,PORT=12345`

    If you pass any variable to upstart on initialization, the same variables
    must be passed to restart.
    """
    args = ' '.join(['{}={}'.format(k, v) for k, v in kwargs.iteritems()])

    cmd = 'restart'
    if force_start:
        with fab.settings(warn_only=True):
            status = fab.sudo("status {name} {args}".format(name=name, args=args))
        if status.failed:
            cmd = 'start'
    fab.sudo("{cmd} {name} {args}".format(cmd=cmd, name=name, args=args))


@fab.task
def disable(name):
    """
    Disable specified upstart job
    """
    fab.sudo("echo 'manual' | tee /etc/init/{}.override".format(name))



# coding: utf-8
import os
from functools import partial

import cuisine
import fabric.api as fab

from fabix import get_config

_INSTALL_DIR = '/opt'
_DOWNLOAD_URL = 'http://nginx.org/download/nginx-{version}.tar.gz'
NGINX_USER = 'nginx'
ETC_DIR = os.path.join(os.path.dirname(__file__), 'support_files', 'etc')


get_config = partial(get_config, 'nginx')


@fab.task
def install(force=False):
    """Install nginx HTTP server."""
    version = get_config()['version']

    cuisine.package_install(['build-essential', 'libpcre3-dev', 'zlib1g-dev'])

    install_dir = os.path.join(_INSTALL_DIR, 'nginx', version)
    nginx_bin = os.path.join(install_dir, 'sbin', 'nginx')
    if cuisine.file_exists(nginx_bin):
        if not force:
            fab.puts("Nginx {0} found, skipping installation".format(version))
            return
        else:
            fab.puts("Reinstalling nginx {0} found".format(version))

    with cuisine.mode_sudo():
        cuisine.dir_ensure(install_dir, True)

    home_dir = os.path.join(install_dir, 'html')
    cuisine.user_ensure(NGINX_USER, None, home_dir, shell='/sbin/nologin')
    fab.sudo('passwd -l nginx')

    download_url = _DOWNLOAD_URL.format(version=version)

    src_dir = fab.run('mktemp -d')
    with fab.cd(src_dir):
        fab.puts("Downloading nginx {0}".format(version))
        fab.run("wget -q '{0}' -O - | tar xz".format(download_url))
        with fab.cd('nginx-{0}'.format(version)):
            fab.puts("Compiling nginx {0}".format(version))
            fab.run("./configure --prefix={0} --with-http_stub_status_module".format(install_dir))
            fab.run("make")
            fab.puts("Installing nginx {0}".format(version))
            fab.sudo('make install')
            with cuisine.mode_sudo():
                cuisine.dir_ensure("{0}{1}".format(install_dir, '/conf/sites-enabled'))
    fab.run("rm -rf '{0}'".format(src_dir))


@fab.task
def uninstall():
    """Uninstall nginx HTTP server"""
    version = get_config()['version']

    install_dir = os.path.join(_INSTALL_DIR, 'nginx')
    if version != 'all':
        install_dir = os.path.join(install_dir, version)

    fab.puts("Removing {0}".format(install_dir))
    if fab.confirm("Are you sure?", default=False):
        fab.sudo("rm -rf '{0}'".format(install_dir))
        fab.puts("Nginx {0} uninstalled".format(version))


@fab.task
def install_upstart():
    """Install nginx upstart config."""
    version = get_config()['version']

    install_dir = os.path.join(_INSTALL_DIR, 'nginx', version)
    nginx_bin = os.path.join(install_dir, 'sbin', 'nginx')
    nginx_pid = os.path.join(install_dir, 'logs', 'nginx.pid')

    context = {
        'nginx_bin': nginx_bin,
        'nginx_pid': nginx_pid,
    }

    nginx_tpl = os.path.join(ETC_DIR, 'init', 'nginx.conf')
    tpl_content = cuisine.file_local_read(nginx_tpl)
    content = cuisine.text_template(tpl_content, context)

    with cuisine.mode_sudo():
        cuisine.file_write('/etc/init/nginx.conf', content)


@fab.task
def put_conf(nginx_file):
    """Install global nginx config."""
    version = get_config()['version']

    install_dir = os.path.join(_INSTALL_DIR, 'nginx', version)
    conf_file = os.path.join(install_dir, 'conf', 'nginx.conf')

    if not os.path.exists(nginx_file):
        fab.abort("Nginx conf {0} not found".format(nginx_file))

    nginx_pid = os.path.join(install_dir, 'logs', 'nginx.pid')

    context = {
        'nginx_user': NGINX_USER,
        'nginx_pid': nginx_pid,
    }

    tpl_content = open(nginx_file, 'rb').read()
    content = cuisine.text_template(tpl_content, context)

    with cuisine.mode_sudo():
        cuisine.file_write(conf_file, content)


@fab.task
def put_site_conf(nginx_file, context=None):
    """Install nginx config per site."""
    version = get_config()['version']

    if not os.path.exists(nginx_file):
        fab.abort("Nginx conf {0} not found".format(nginx_file))

    site_name = os.path.basename(nginx_file)

    install_dir = os.path.join(_INSTALL_DIR, 'nginx', version)
    conf_file = os.path.join(install_dir, 'conf', 'sites-enabled', site_name)

    if context:
        tpl_content = open(nginx_file, 'rb').read()
        content = cuisine.text_template(tpl_content, context)

        with cuisine.mode_sudo():
            cuisine.file_write(conf_file, content)
    else:
        with cuisine.mode_sudo():
            cuisine.file_upload(conf_file, nginx_file)


@fab.task
def setup(nginx_file, nginx_site_conf):
    """Installs and configures nginx"""
    install()
    install_upstart()
    put_conf(nginx_file)
    put_site_conf(nginx_site_conf)



# coding: utf-8
import fabric.api as fab
import cuisine


def install(filename, user="root"):
    """
    Installs crontab from a given cronfile
    """
    tmp_file = fab.run("mktemp fabixcron")
    cuisine.file_upload(tmp_file, filename)
    fab.sudo("crontab -u{} {}".format(user, tmp_file))



import os
from functools import partial

import cuisine
import fabric.api as fab

from fabix import get_config, get_project_name

_INSTALL_DIR = '/opt'
PYTHON_DOWNLOAD_URL = 'http://www.python.org/ftp/python/{version}/Python-{version}.tgz'
SETUPTOOLS_DOWNLOAD_URL = 'http://pypi.python.org/packages/source/s/setuptools/setuptools-0.6c11.tar.gz'
SITES_DIR = '/data/sites/'


get_proj_config = get_config
get_config = partial(get_config, 'python')


@fab.task
def install(force=False):
    """Install python"""
    version = get_config()['version']

    install_dir = os.path.join(_INSTALL_DIR, 'python', version)
    python_bin = os.path.join(install_dir, 'bin', 'python')

    if cuisine.file_exists(python_bin):
        if not force:
            fab.puts("Python {0} found, skipping installation".format(version))
            return
        else:
            fab.puts("Reinstalling Python {0} found".format(version))

    fab.package_install(['build-essential', 'libcurl4-openssl-dev'])

    src_dir = fab.run('mktemp -d')
    with fab.cd(src_dir):
        fab.puts("Downloading python {0}".format(version))
        fab.run("wget -q '%s' -O - | tar xz" % PYTHON_DOWNLOAD_URL.format(version=version))
        with fab.cd('Python-{0}'.format(version)):
            fab.puts("Installing python {0}".format(version))
            fab.run("./configure --prefix=%s" % install_dir)
            fab.run("make")
            fab.sudo('make install')
    fab.run('rm -rf {0}'.format(src_dir))


def _python_bin_path(py_version, bin_name='python'):
    install_dir = os.path.join(_INSTALL_DIR, 'python', py_version)
    return os.path.join(install_dir, 'bin', bin_name)


@fab.task
def install_setuptools(force=False):
    """Install setuptools"""
    py_version = get_config()['version']

    easy_install_bin = _python_bin_path(py_version, 'easy_install')
    if cuisine.file_exists(easy_install_bin):
        if not force:
            fab.puts("easy_install for python {0} found, skipping installation".format(py_version))
            return
        else:
            fab.puts("Reinstalling easy_install for python {0}".format(py_version))

    major, minor = py_version.split('.')[0:2]
    version = "{0}.{1}".format(major, minor)

    python_bin = _python_bin_path(py_version)

    src_dir = fab.run('mktemp -d')
    with fab.cd(src_dir):
        fab.puts("Downloading setuptools for python {0}".format(version))
        download_url = SETUPTOOLS_DOWNLOAD_URL.format(py_version=version)
        fab.run("wget -q '{0}' -O - | tar xz".format(download_url))
        with fab.cd('setuptools-*'):
            fab.puts("Installing setuptools for python {0}".format(version))
            fab.sudo("{0} setup.py install".format(python_bin))
    fab.sudo('rm -rf {0}'.format(src_dir))


@fab.task
def uninstall_setuptools():
    """Uninstall setuptools"""
    py_version = get_config()['version']

    major, minor = py_version.split('.')[0:2]
    version = "{0}.{1}".format(major, minor)

    python_bin = _python_bin_path(py_version)

    src_dir = fab.run('mktemp -d')
    with fab.cd(src_dir):
        fab.puts("Downloading setuptools for python {0}".format(version))
        download_url = SETUPTOOLS_DOWNLOAD_URL.format(py_version=version)
        fab.run("wget -q '{0}' -O - | tar xz".format(download_url))
        with fab.cd('setuptools-*'):
            fab.puts("Uninstalling setuptools for python {0}".format(version))
            fab.sudo("{0} setup.py install --record setuptools_files.txt".format(python_bin))
            fab.sudo("cat setuptools_files.txt | xargs rm -rf")
    fab.sudo('rm -rf {0}'.format(src_dir))


@fab.task
def uninstall():
    """Uninstall python"""
    version = get_config()['version']

    install_dir = os.path.join(_INSTALL_DIR, 'python')
    if version != 'all':
        install_dir = os.path.join(install_dir, version)

    fab.puts("Removing {0}".format(install_dir))
    if fab.confirm("Are you sure?", default=False):
        fab.sudo("rm -rf '{0}'".format(install_dir))
        fab.puts("Python {0} uninstalled".format(version))


@fab.task
def install_pip():
    """Install pip latest version."""
    py_version = get_config()['version']

    fab.puts("Installing pip for python {0}".format(py_version))
    easy_install_bin = _python_bin_path(py_version, 'easy_install')

    if not cuisine.file_exists(easy_install_bin):
        fab.puts("easy_install for version {0} not found".format(py_version))
        return

    fab.sudo('{0} pip'.format(easy_install_bin))


@fab.task
def install_pypi_package(package, use_virtualenv=True):
    """Install pypi package `package` on python `py_version`."""
    py_version = get_config()['version']

    fab.puts("Installing pypi package {0} on python {1}".format(package, py_version))

    if use_virtualenv:
        site = get_project_name()
        pip_bin = _get_virtualenv_bin(site, 'pip')
    else:
        pip_bin = _python_bin_path(py_version, 'pip')

    if not cuisine.file_exists(pip_bin):
        fab.puts("pip for version {0} not found".format(py_version))
        return

    fab.sudo('{cmd} install {package}'.format(cmd=pip_bin, package=package))


@fab.task
def uninstall_pypi_package(package, use_virtualenv=True):
    """Uninstall pypi package `package` on python `py_version`."""
    py_version = get_config()['version']

    fab.puts("Uninstalling pypi package {0} on python {1}".format(package, py_version))

    if use_virtualenv:
        site = get_project_name()
        pip_bin = _get_virtualenv_bin(site, 'pip')
    else:
        pip_bin = _python_bin_path(py_version, 'pip')

    if not cuisine.file_exists(pip_bin):
        fab.puts("pip for version {0} not found".format(py_version))
        return

    fab.sudo('{cmd} uninstall --yes {package}'.format(cmd=pip_bin, package=package))


@fab.task
def create_virtualenv():
    """Create virtualenv for project."""
    site = get_project_name()
    version = get_config()['version']

    virtualenv_dir = "{}/{}/virtualenv".format(SITES_DIR, site)
    if cuisine.dir_exists(virtualenv_dir):
        fab.puts("virtualenv for {0} already exists".format(site))
        return

    with cuisine.mode_sudo():
        cuisine.dir_ensure(virtualenv_dir, recursive=True)

    venv_bin = _python_bin_path(version, 'virtualenv')
    fab.sudo("{venv_bin} {virtualenv_dir}".format(venv_bin=venv_bin,
             virtualenv_dir=virtualenv_dir))


def _get_virtualenv_bin(site, binary):
    """Grabs a binary in a given virtualenv"""
    bin_dir = os.path.join(SITES_DIR, site, 'virtualenv', 'bin')
    return os.path.join(bin_dir, binary)


@fab.task
def install_requirements(upgrade=False):
    """Install `site` project's requirements"""
    site = get_project_name()
    project_dir = get_proj_config()['project_dir']

    pip = _get_virtualenv_bin(site, 'pip')
    requirements = open("{}/requirements.txt".format(project_dir)).read().replace("\n", " ")
    fab.sudo("{pip} install {upgrade} {requirements}".format(
             pip=pip, upgrade="--upgrade" if upgrade else "",
             requirements=requirements))


@fab.task
def setup():
    """Install python, setuptools, pip and virtualenv."""
    install()
    install_setuptools()
    install_pip()
    install_pypi_package('virtualenv')



# coding: utf-8
import fabric.api as fab

VERSION = (0, 0, 2)

__version__ = ".".join([str(v) for v in VERSION])

fab.env.fabix = fab.env.get('fabix', dict())


def get_project_name():
    fab.require('fabix')
    name = fab.env.fabix.get('_current_project', None)
    assert name, "No project defined."
    return name


def get_config(name=None):
    fab.require('fabix')
    pj_name = get_project_name()
    config = fab.env.fabix.get(pj_name, dict())
    if name is not None:
        return config.get(name)
    else:
        return config



# coding: utf-8
import time

import fabric.api as fab

from fabix.system import upstart


@fab.task
def ssh():
    fab.puts("Opening shell to host {0}".format(fab.env.host_string))
    fab.open_shell()


@fab.task
def apt_import_pubkey(key, keyserver='keyserver.ubuntu.com'):
    fab.puts("Importing pubkey {0}".format(key))
    fab.sudo('apt-key adv --keyserver {0} --recv {1}'.format(keyserver, key))


@fab.task
def apt_add_repository(name, distro, url):
    fab.puts("Add repository {0}".format(name))
    fab.sudo("echo 'deb {0} {1} {2}' | sudo tee /etc/apt/sources.list.d/{2}.list".format(url, distro, name))


@fab.task
def reboot():
    fab.puts("Rebooting machine in 5 seconds!")
    # just some time so user can press Ctrl+C
    time.sleep(5)
    fab.sudo('reboot')



import os

import boto
from boto.utils import compute_md5
from fabric.decorators import task
from fabric.utils import puts


def upload_file(bucket, key_name, file_path, remote_prefix=None, policy='public-read'):
    if remote_prefix:
        key_name = '{0}/{1}'.format(remote_prefix, key_name)
    key = bucket.new_key(key_name)
    fd = open(file_path)
    md5 = compute_md5(fd)
    fd.close()
    key.set_metadata('fabix-md5', md5[0])
    key.set_contents_from_filename(file_path, md5=md5, policy=policy)
    return key


def get_key_name(local_path, fullpath):
    key_name = fullpath[len(local_path):]
    l = key_name.split(os.sep)
    key_name = '/'.join(l)
    return key_name.lstrip('/')


@task
def sync_dir_up(bucket_name, local_path, remote_prefix=None):
    puts("Sync directory {0} with bucket {1}".format(bucket_name, local_path))
    conn = boto.connect_s3()
    bucket = conn.get_bucket(bucket_name)

    for root, dirs, files in os.walk(local_path):
        for fname in files:
            file_path = os.path.join(root, fname)
            key_name = get_key_name(local_path, file_path)

            key = bucket.lookup(key_name)
            if key:
                key_md5 = key.get_metadata('fabix-md5')
                fd = open(file_path)
                md5 = compute_md5(fd)
                fd.close()
                if md5[0] == key_md5:
                    puts("Skipping {0} (MD5 match)".format(file_path))
                    continue

            puts("Upload file {0}".format(file_path))
            upload_file(bucket, key_name, file_path, remote_prefix=remote_prefix)



import os

from setuptools import setup, find_packages
from fabix import __version__

base_dir = os.path.dirname(__file__)
readme_file = os.path.join(base_dir, 'README.rst')
requirements_file = os.path.join(base_dir, 'requirements.txt')

setup(
    name='fabix',
    version=__version__,
    description="Fabix is a serie of functions built on top of fabric and cuisine to easily deploy python web projects.",
    long_description=open(readme_file, 'rb').read(),
    keywords=['fabric', 'cuisine', 'fabix'],
    author='Rodrigo Machado',
    author_email='rcmachado@gmail.com',
    url='http://github.com/quatix/fabix',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Systems Administration',
    ],
    packages=find_packages(),
    install_requires=open(requirements_file, "rb").read().split("\n"),
    package_dir={"fabix": "fabix"}
)



import os
from datetime import datetime
from tempfile import mkdtemp

import cuisine
import fabric.api as fab

from fabix import get_config

INSTALL_DIR = '/data/sites'


class env(object):
    def __init__(self, project_name):
        self._old_env_name = None
        self._current_env_name = project_name
        if 'fabix' in fab.env:
            self._old_env_name = fab.env.get('fabix', dict()).get('_current_project', None)

    def __enter__(self):
        fab.env.fabix['_current_project'] = self._current_env_name

    def __exit__(self, type, value, traceback):
        if self._old_env_name is not None:
            fab.env.fabix['_current_project'] = self._old_env_name


@fab.task
def upload(tag='master'):
    """Upload project `site` files from tag or branch `master`."""
    site = fab.env.fabix['_current_project']
    project_dir = get_config()['project_dir']

    install_dir = os.path.join(INSTALL_DIR, site)
    dirs_to_create = (
        os.path.join(install_dir, 'releases'),
    )
    fab.sudo('mkdir -p {0}'.format(' '.join(dirs_to_create)))

    fab.puts("Upload project {0}".format(site))

    today = datetime.now().strftime('%Y%m%d-%H%M%S')
    commit_id = str(fab.local('git rev-parse {0}'.format(tag), True)).strip()
    current = "%s-%s" % (today, commit_id[:8])
    release_dir = os.path.join(INSTALL_DIR, site, 'releases')

    with cuisine.mode_sudo():
        cuisine.dir_ensure(release_dir)

    local_temp_dir = mkdtemp()
    archive = os.path.join(local_temp_dir, '{0}.tar.gz'.format(site))

    git_arch_cmd = "git archive --format=tar.gz -o {archive} {commit_id}:{project_dir}"
    fab.local(git_arch_cmd.format(archive=archive, commit_id=commit_id, project_dir=project_dir))

    with fab.lcd(local_temp_dir):
        remote_temp_dir = fab.run('mktemp -d')
        fab.put(archive, remote_temp_dir)
        with fab.cd(remote_temp_dir):
            fab.run("tar xzf {0}.tar.gz".format(site))
            fab.run("rm -f {0}.tar.gz".format(site))
        fab.sudo('chown -R root.root {0}'.format(remote_temp_dir))
        fab.sudo('chmod -R 755 {0}'.format(remote_temp_dir))
        fab.sudo("mv {tmp_dir} {release_dir}/{current}".format(release_dir=release_dir, current=current, tmp_dir=remote_temp_dir))
        fab.run('rm -rf {0}'.format(remote_temp_dir))
    fab.local('rm -rf {0}'.format(local_temp_dir))

    return current


@fab.task
def activate(release):
    """Activate release `release` code for project `project`."""
    project = fab.env.fabix['_current_project']

    fab.puts("Activating project {0} release {1}".format(project, release))
    with fab.cd(os.path.join(INSTALL_DIR, project)):
        fab.sudo("ln -nsf releases/{release} {project}".format(project=project, release=release))


@fab.task
def cleanup(keep=5):
    """Cleanup old releases for `site` keeping the `keep` most recent."""
    site = fab.env.fabix['_current_project']

    with fab.cd(os.path.join(INSTALL_DIR, site, 'releases')):
        files_to_remove = str(fab.run('ls -1')).split()
        if files_to_remove:
            cmd_args = ' '.join(files_to_remove[:-keep])
            fab.puts("Removing releases {0}".format(cmd_args))
            fab.sudo('rm -rf {0}'.format(cmd_args))
